/*	$NetBSD$	*/

/*
 * Copyright (c) 2003 Shigeyuki Fukushima.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Shigeyuki Fukushima
 *	for the NetBSD Project.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "locators.h"

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/device.h>

#include <machine/bus.h>

#include <powerpc/ibm4xx/dcr405gp.h>
#include <powerpc/ibm4xx/ibm405gp.h>
#include <powerpc/ibm4xx/dev/opbvar.h>
#include <powerpc/ibm4xx/dev/gpioreg.h>
#include <powerpc/ibm4xx/dev/gpiovar.h>

static int   gpio_match(struct device *, struct cfdata *, void *);
static void  gpio_attach(struct device *, struct device *, void *);
static int   gpio_submatch(struct device *parent, struct cfdata *cf, void *aux);
static int   gpio_print(void *aux, const char *pnp);

CFATTACH_DECL(gpio, sizeof(struct device),
    gpio_match, gpio_attach, NULL, NULL);

/*
 * Probe for the GPIO.
 */
static int
gpio_match(struct device *parent, struct cfdata *cf, void *aux)
{
	struct opb_attach_args *oaa = aux;

	/* match only on-chip iic devices */
	if (strcmp(oaa->opb_name, cf->cf_name) == 0)
		return (1);

	return (0);
}

static int
gpio_submatch(struct device *parent, struct cfdata *cf, void *aux)
{
	struct gpio_attach_args *gaa = aux;

	if (cf->cf_loc[GPIOCF_ADDR] != GPIOCF_ADDR_DEFAULT &&
	    cf->cf_loc[GPIOCF_ADDR] != gaa->sccaddr)
		return (0);

	return (config_match(parent, cf, aux));
}

/*
 * Attach the GPIO.
 */
static void
gpio_attach(struct device *parent, struct device *self, void *aux)
{
	struct opb_attach_args *oaa = aux;
	struct gpio_attach_args *gaa;

	gaa->gpio_addr = oaa->opb_addr;
	gaa->gpio_irq = GPIOCF_IRQ_DEFAULT;
	gaa->gpio_bt = oaa->opb_bt;
	gaa->gpio_dmat = oaa->opb_dmat;

	printf(": 405GP GPIO\n");

	return;
}

static int
gpio_print(void *aux, const char *pnp)
{
	struct gpio_attach_args *gaa = aux;
	if (pnp)
		printf("%s at %s", gaa->gpio_name, pnp);
	if (gaa->gpio_addr != GPIOCF_ADDR_DEFAULT)
		aprint_normal(" addr 0x%08lx", gaa->gpio_addr);
	if (gaa->gpio_irq != GPIOCF_IRQ_DEFAULT)
		aprint_normal(" irq %d", gaa->gpio_irq);

	return (UNCONF);
}
